@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2012 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -54,7 +54,6 @@
 #define TEMPEXPR_DELTA 8
 
 static scanner scan;            /* Address of scanner routine */
-static efunc error;             /* Address of error reporting routine */
 static lfunc labelfunc;         /* Address of label routine */
 
 static struct ofmt *outfmt;     /* Structure of addresses of output routines */
@@ -148,8 +147,11 @@ static expr *add_vectors(expr * p, expr * q)
             lasttype = p++->type;
         } else {                /* *p and *q have same type */
             int64_t sum = p->value + q->value;
-            if (sum)
+            if (sum) {
                 addtotemp(p->type, sum);
+                if (hint)
+                    hint->type = EAH_SUMMED;
+            }
             lasttype = p->type;
             p++, q++;
         }
@@ -224,18 +226,18 @@ static expr *segment_part(expr * e)
         return unknown_expr();
 
     if (!is_reloc(e)) {
-        error(ERR_NONFATAL, "cannot apply SEG to a non-relocatable value");
+        nasm_error(ERR_NONFATAL, "cannot apply SEG to a non-relocatable value");
         return NULL;
     }
 
     seg = reloc_seg(e);
     if (seg == NO_SEG) {
-        error(ERR_NONFATAL, "cannot apply SEG to a non-relocatable value");
+        nasm_error(ERR_NONFATAL, "cannot apply SEG to a non-relocatable value");
         return NULL;
     } else if (seg & SEG_ABS) {
         return scalarvect(seg & ~SEG_ABS);
     } else if (seg & 1) {
-        error(ERR_NONFATAL, "SEG applied to something which"
+        nasm_error(ERR_NONFATAL, "SEG applied to something which"
               " is already a segment base");
         return NULL;
     } else {
@@ -274,7 +276,7 @@ static expr *segment_part(expr * e)
  * expr3 : expr4 [ {<<,>>} expr4...]
  * expr4 : expr5 [ {+,-} expr5...]
  * expr5 : expr6 [ {*,/,%,//,%%} expr6...]
- * expr6 : { ~,+,-,SEG } expr6
+ * expr6 : { ~,+,-,IFUNC,SEG } expr6
  *       | (bexpr)
  *       | symbol
  *       | $
@@ -303,7 +305,7 @@ static expr *rexp0(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`|' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`|' operator may only be applied to"
                   " scalar values");
         }
 
@@ -330,7 +332,7 @@ static expr *rexp1(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`^' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`^' operator may only be applied to"
                   " scalar values");
         }
 
@@ -356,7 +358,7 @@ static expr *rexp2(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`&' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`&' operator may only be applied to"
                   " scalar values");
         }
         if (is_just_unknown(e) || is_just_unknown(f))
@@ -400,7 +402,7 @@ static expr *rexp3(int critical)
             if (is_unknown(e))
                 v = -1;         /* means unknown */
             else if (!is_really_simple(e)) {
-                error(ERR_NONFATAL,
+                nasm_error(ERR_NONFATAL,
                       "`%s': operands differ by a non-scalar",
                       (j == TOKEN_LE ? "<=" : j == TOKEN_LT ? "<" : j ==
                        TOKEN_GE ? ">=" : ">"));
@@ -440,7 +442,7 @@ static expr *expr0(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`|' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`|' operator may only be applied to"
                   " scalar values");
         }
         if (is_just_unknown(e) || is_just_unknown(f))
@@ -466,7 +468,7 @@ static expr *expr1(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`^' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`^' operator may only be applied to"
                   " scalar values");
         }
         if (is_just_unknown(e) || is_just_unknown(f))
@@ -492,7 +494,7 @@ static expr *expr2(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "`&' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`&' operator may only be applied to"
                   " scalar values");
         }
         if (is_just_unknown(e) || is_just_unknown(f))
@@ -519,7 +521,7 @@ static expr *expr3(int critical)
             return NULL;
         if (!(is_simple(e) || is_just_unknown(e)) ||
             !(is_simple(f) || is_just_unknown(f))) {
-            error(ERR_NONFATAL, "shift operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "shift operator may only be applied to"
                   " scalar values");
         } else if (is_just_unknown(e) || is_just_unknown(f)) {
             e = unknown_expr();
@@ -578,12 +580,12 @@ static expr *expr5(int critical)
             return NULL;
         if (j != '*' && (!(is_simple(e) || is_just_unknown(e)) ||
                          !(is_simple(f) || is_just_unknown(f)))) {
-            error(ERR_NONFATAL, "division operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "division operator may only be applied to"
                   " scalar values");
             return NULL;
         }
         if (j != '*' && !is_unknown(f) && reloc_value(f) == 0) {
-            error(ERR_NONFATAL, "division by zero");
+            nasm_error(ERR_NONFATAL, "division by zero");
             return NULL;
         }
         switch (j) {
@@ -595,7 +597,7 @@ static expr *expr5(int critical)
             else if (is_just_unknown(e) && is_just_unknown(f))
                 e = unknown_expr();
             else {
-                error(ERR_NONFATAL, "unable to multiply two "
+                nasm_error(ERR_NONFATAL, "unable to multiply two "
                       "non-scalar objects");
                 return NULL;
             }
@@ -654,7 +656,7 @@ static expr *eval_floatize(enum floatize type)
 
     i = scan(scpriv, tokval);
     if (i != '(') {
-        error(ERR_NONFATAL, "expecting `('");
+        nasm_error(ERR_NONFATAL, "expecting `('");
         return NULL;
     }
     i = scan(scpriv, tokval);
@@ -663,15 +665,14 @@ static expr *eval_floatize(enum floatize type)
         i = scan(scpriv, tokval);
     }
     if (i != TOKEN_FLOAT) {
-        error(ERR_NONFATAL, "expecting floating-point number");
+        nasm_error(ERR_NONFATAL, "expecting floating-point number");
         return NULL;
     }
-    if (!float_const(tokval->t_charptr, sign, result,
-                     formats[type].bytes, error))
+    if (!float_const(tokval->t_charptr, sign, result, formats[type].bytes))
         return NULL;
     i = scan(scpriv, tokval);
     if (i != ')') {
-        error(ERR_NONFATAL, "expecting `)'");
+        nasm_error(ERR_NONFATAL, "expecting `)'");
         return NULL;
     }
 
@@ -703,13 +704,13 @@ static expr *eval_strfunc(enum strfunc type)
         i = scan(scpriv, tokval);
     }
     if (i != TOKEN_STR) {
-        error(ERR_NONFATAL, "expecting string");
+        nasm_error(ERR_NONFATAL, "expecting string");
         return NULL;
     }
     string_len = string_transform(tokval->t_charptr, tokval->t_inttwo,
                                   &string, type);
     if (string_len == (size_t)-1) {
-        error(ERR_NONFATAL, "invalid string for transform");
+        nasm_error(ERR_NONFATAL, "invalid string for transform");
         return NULL;
     }
 
@@ -717,13 +718,13 @@ static expr *eval_strfunc(enum strfunc type)
     if (parens) {
         i = scan(scpriv, tokval);
         if (i != ')') {
-            error(ERR_NONFATAL, "expecting `)'");
+            nasm_error(ERR_NONFATAL, "expecting `)'");
             return NULL;
         }
     }
 
     if (rn_warn)
-        error(ERR_WARNING|ERR_PASS1, "character constant too long");
+        nasm_error(ERR_WARNING|ERR_PASS1, "character constant too long");
 
     begintemp();
     addtotemp(EXPR_SIMPLE, val);
@@ -732,6 +733,37 @@ static expr *eval_strfunc(enum strfunc type)
     return finishtemp();
 }
 
+static int64_t eval_ifunc(int64_t val, enum ifunc func)
+{
+    int errtype;
+    uint64_t uval = (uint64_t)val;
+    int64_t rv;
+
+    switch (func) {
+    case IFUNC_ILOG2E:
+    case IFUNC_ILOG2W:
+        errtype = (func == IFUNC_ILOG2E) ? ERR_NONFATAL : ERR_WARNING;
+
+        if (!is_power2(uval))
+            nasm_error(errtype, "ilog2 argument is not a power of two");
+        /* fall through */
+    case IFUNC_ILOG2F:
+        rv = ilog2_64(uval);
+        break;
+
+    case IFUNC_ILOG2C:
+        rv = (uval < 2) ? 0 : ilog2_64(uval-1) + 1;
+        break;
+
+    default:
+        nasm_panic(0, "invalid IFUNC token %d", func);
+        rv = 0;
+        break;
+    }
+
+    return rv;
+}
+
 static expr *expr6(int critical)
 {
     int32_t type;
@@ -762,7 +794,7 @@ static expr *expr6(int critical)
         if (is_just_unknown(e))
             return unknown_expr();
         else if (!is_simple(e)) {
-            error(ERR_NONFATAL, "`~' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`~' operator may only be applied to"
                   " scalar values");
             return NULL;
         }
@@ -776,12 +808,29 @@ static expr *expr6(int critical)
         if (is_just_unknown(e))
             return unknown_expr();
         else if (!is_simple(e)) {
-            error(ERR_NONFATAL, "`!' operator may only be applied to"
+            nasm_error(ERR_NONFATAL, "`!' operator may only be applied to"
                   " scalar values");
             return NULL;
         }
         return scalarvect(!reloc_value(e));
 
+    case TOKEN_IFUNC:
+    {
+        enum ifunc func = tokval->t_integer;
+        i = scan(scpriv, tokval);
+        e = expr6(critical);
+        if (!e)
+            return NULL;
+        if (is_just_unknown(e))
+            return unknown_expr();
+        else if (!is_simple(e)) {
+            nasm_error(ERR_NONFATAL, "function may only be applied to"
+                  " scalar values");
+            return NULL;
+        }
+        return scalarvect(eval_ifunc(reloc_value(e), func));
+    }
+
     case TOKEN_SEG:
         i = scan(scpriv, tokval);
         e = expr6(critical);
@@ -791,7 +840,7 @@ static expr *expr6(int critical)
         if (!e)
             return NULL;
         if (is_unknown(e) && critical) {
-            error(ERR_NONFATAL, "unable to determine segment base");
+            nasm_error(ERR_NONFATAL, "unable to determine segment base");
             return NULL;
         }
         return e;
@@ -808,7 +857,7 @@ static expr *expr6(int critical)
         if (!e)
             return NULL;
         if (i != ')') {
-            error(ERR_NONFATAL, "expecting `)'");
+            nasm_error(ERR_NONFATAL, "expecting `)'");
             return NULL;
         }
         i = scan(scpriv, tokval);
@@ -821,6 +870,7 @@ static expr *expr6(int critical)
     case TOKEN_INSN:            /* Opcodes that occur here are really labels */
     case TOKEN_HERE:
     case TOKEN_BASE:
+    case TOKEN_DECORATOR:
         begintemp();
         switch (i) {
         case TOKEN_NUM:
@@ -829,7 +879,7 @@ static expr *expr6(int critical)
         case TOKEN_STR:
             tmpval = readstrnum(tokval->t_charptr, tokval->t_inttwo, &rn_warn);
             if (rn_warn)
-                error(ERR_WARNING|ERR_PASS1, "character constant too long");
+                nasm_error(ERR_WARNING|ERR_PASS1, "character constant too long");
             addtotemp(EXPR_SIMPLE, tmpval);
             break;
         case TOKEN_REG:
@@ -847,7 +897,7 @@ static expr *expr6(int critical)
              * are in preprocess-only mode.
              */
             if (!location->known) {
-                error(ERR_NONFATAL,
+                nasm_error(ERR_NONFATAL,
                       "%s not supported in preprocess-only mode",
                       (i == TOKEN_HERE ? "`$'" :
                        i == TOKEN_BASE ? "`$$'" :
@@ -867,11 +917,11 @@ static expr *expr6(int critical)
                 if (!labelfunc(tokval->t_charptr, &label_seg, &label_ofs)) {
                     scope = local_scope(tokval->t_charptr);
                     if (critical == 2) {
-                        error(ERR_NONFATAL, "symbol `%s%s' undefined",
+                        nasm_error(ERR_NONFATAL, "symbol `%s%s' undefined",
                               scope,tokval->t_charptr);
                         return NULL;
                     } else if (critical == 1) {
-                        error(ERR_NONFATAL,
+                        nasm_error(ERR_NONFATAL,
                               "symbol `%s%s' not defined before use",
                               scope,tokval->t_charptr);
                         return NULL;
@@ -890,12 +940,15 @@ static expr *expr6(int critical)
             if (label_seg != NO_SEG)
                 addtotemp(EXPR_SEGBASE + label_seg, 1L);
             break;
+        case TOKEN_DECORATOR:
+            addtotemp(EXPR_RDSAE, tokval->t_integer);
+            break;
         }
         i = scan(scpriv, tokval);
         return finishtemp();
 
     default:
-        error(ERR_NONFATAL, "expression syntax error");
+        nasm_error(ERR_NONFATAL, "expression syntax error");
         return NULL;
     }
 }
@@ -909,8 +962,7 @@ void eval_global_info(struct ofmt *output, lfunc lookup_label,
 }
 
 expr *evaluate(scanner sc, void *scprivate, struct tokenval *tv,
-               int *fwref, int critical, efunc report_error,
-               struct eval_hints *hints)
+               int *fwref, int critical, struct eval_hints *hints)
 {
     expr *e;
     expr *f = NULL;
@@ -928,7 +980,6 @@ expr *evaluate(scanner sc, void *scprivate, struct tokenval *tv,
     scan = sc;
     scpriv = scprivate;
     tokval = tv;
-    error = report_error;
     opflags = fwref;
 
     if (tokval->t_type == TOKEN_INVALID)
@@ -958,14 +1009,14 @@ expr *evaluate(scanner sc, void *scprivate, struct tokenval *tv,
             int64_t value;
             begintemp();
             if (!is_reloc(f)) {
-                error(ERR_NONFATAL, "invalid right-hand operand to WRT");
+                nasm_error(ERR_NONFATAL, "invalid right-hand operand to WRT");
                 return NULL;
             }
             value = reloc_seg(f);
             if (value == NO_SEG)
                 value = reloc_value(f) | SEG_ABS;
             else if (!(value & SEG_ABS) && !(value % 2) && critical) {
-                error(ERR_NONFATAL, "invalid right-hand operand to WRT");
+                nasm_error(ERR_NONFATAL, "invalid right-hand operand to WRT");
                 return NULL;
             }
             addtotemp(EXPR_WRT, value);
