@@ -106,7 +106,7 @@ enum out_type {
     OUT_REL1ADR,    /* 1-byte relative address */
     OUT_REL2ADR,    /* 2-byte relative address */
     OUT_REL4ADR,    /* 4-byte relative address */
-    OUT_REL8ADR,    /* 8-byte relative address */
+    OUT_REL8ADR     /* 8-byte relative address */
 };
 
 /*
@@ -126,70 +126,6 @@ typedef void (*ldfunc)(char *label, int32_t segment, int64_t offset,
 void define_label(char *label, int32_t segment, int64_t offset,
                   char *special, bool is_norm, bool isextrn);
 
-/*
- * List-file generators should look like this:
- */
-typedef struct {
-    /*
-     * Called to initialize the listing file generator. Before this
-     * is called, the other routines will silently do nothing when
-     * called. The `char *' parameter is the file name to write the
-     * listing to.
-     */
-    void (*init)(char *fname, efunc error);
-
-    /*
-     * Called to clear stuff up and close the listing file.
-     */
-    void (*cleanup)(void);
-
-    /*
-     * Called to output binary data. Parameters are: the offset;
-     * the data; the data type. Data types are similar to the
-     * output-format interface, only OUT_ADDRESS will _always_ be
-     * displayed as if it's relocatable, so ensure that any non-
-     * relocatable address has been converted to OUT_RAWDATA by
-     * then. Note that OUT_RAWDATA,0 is a valid data type, and is a
-     * dummy call used to give the listing generator an offset to
-     * work with when doing things like uplevel(LIST_TIMES) or
-     * uplevel(LIST_INCBIN).
-     */
-    void (*output)(int32_t offset, const void *data, enum out_type type, uint64_t size);
-
-    /*
-     * Called to send a text line to the listing generator. The
-     * `int' parameter is LIST_READ or LIST_MACRO depending on
-     * whether the line came directly from an input file or is the
-     * result of a multi-line macro expansion.
-     */
-    void (*line)(int type, char *line);
-
-    /*
-     * Called to change one of the various levelled mechanisms in
-     * the listing generator. LIST_INCLUDE and LIST_MACRO can be
-     * used to increase the nesting level of include files and
-     * macro expansions; LIST_TIMES and LIST_INCBIN switch on the
-     * two binary-output-suppression mechanisms for large-scale
-     * pseudo-instructions.
-     *
-     * LIST_MACRO_NOLIST is synonymous with LIST_MACRO except that
-     * it indicates the beginning of the expansion of a `nolist'
-     * macro, so anything under that level won't be expanded unless
-     * it includes another file.
-     */
-    void (*uplevel)(int type);
-
-    /*
-     * Reverse the effects of uplevel.
-     */
-    void (*downlevel)(int type);
-
-    /*
-     * Called on a warning or error, with the error message.
-     */
-    void (*error)(int severity, const char *pfx, const char *msg);
-} ListGen;
-
 /*
  * Token types returned by the scanner, in addition to ordinary
  * ASCII character values, and zero for end-of-string.
@@ -228,7 +164,7 @@ enum token_type { /* token types, other than chars */
     TOKEN_STRFUNC,      /* __utf16*__, __utf32*__ */
     TOKEN_IFUNC,        /* __ilog2*__ */
     TOKEN_DECORATOR,    /* decorators such as {...} */
-    TOKEN_OPMASK,       /* translated token for opmask registers */
+    TOKEN_OPMASK        /* translated token for opmask registers */
 };
 
 enum floatize {
@@ -239,7 +175,7 @@ enum floatize {
     FLOAT_80M,
     FLOAT_80E,
     FLOAT_128L,
-    FLOAT_128H,
+    FLOAT_128H
 };
 
 /* Must match the list in string_transform(), in strfunc.c */
@@ -249,14 +185,14 @@ enum strfunc {
     STRFUNC_UTF16BE,
     STRFUNC_UTF32,
     STRFUNC_UTF32LE,
-    STRFUNC_UTF32BE,
+    STRFUNC_UTF32BE
 };
 
 enum ifunc {
     IFUNC_ILOG2E,
     IFUNC_ILOG2W,
     IFUNC_ILOG2F,
-    IFUNC_ILOG2C,
+    IFUNC_ILOG2C
 };
 
 size_t string_transform(char *, size_t, char **, enum strfunc);
@@ -351,7 +287,7 @@ struct eval_hints {
 #define CRITICAL 0x100
 typedef expr *(*evalfunc)(scanner sc, void *scprivate,
                           struct tokenval *tv, int *fwref, int critical,
-                          efunc error, struct eval_hints *hints);
+                          struct eval_hints *hints);
 
 /*
  * Special values for expr->type.
@@ -382,7 +318,7 @@ struct preproc_ops {
      * of the pass, an error reporting function, an evaluator
      * function, and a listing generator to talk to.
      */
-    void (*reset)(char *file, int pass, ListGen *listgen, StrList **deplist);
+    void (*reset)(char *file, int pass, StrList **deplist);
 
     /*
      * Called to fetch a line of preprocessed source. The line
@@ -406,10 +342,13 @@ struct preproc_ops {
 
     /* Include path from command line */
     void (*include_path)(char *path);
+
+    /* Unwind the macro stack when printing an error message */
+    void (*error_list_macros)(int severity);
 };
 
-extern struct preproc_ops nasmpp;
-extern struct preproc_ops preproc_nop;
+extern const struct preproc_ops nasmpp;
+extern const struct preproc_ops preproc_nop;
 
 /*
  * Some lexical properties of the NASM source language, included
@@ -485,6 +424,7 @@ enum ccode { /* condition code names */
 #define TFLAG_BRC_OPT   (1 << 1)    /* may or may not have braces. opmasks {k1} */
 #define TFLAG_BRC_ANY   (TFLAG_BRC | TFLAG_BRC_OPT)
 #define TFLAG_BRDCAST   (1 << 2)    /* broadcasting decorator */
+#define TFLAG_WARN	(1 << 3)    /* warning only, treat as ID */
 
 static inline uint8_t get_cond_opcode(enum ccode c)
 {
@@ -534,7 +474,7 @@ static inline uint8_t get_cond_opcode(enum ccode c)
 enum vex_class {
     RV_VEX      = 0,    /* C4/C5 */
     RV_XOP      = 1,    /* 8F */
-    RV_EVEX     = 2,    /* 62 */
+    RV_EVEX     = 2     /* 62 */
 };
 
 /*
@@ -575,7 +515,7 @@ enum extop_type { /* extended operand types */
     EOT_NOTHING,
     EOT_DB_STRING,      /* Byte string */
     EOT_DB_STRING_FREE, /* Byte string which should be nasm_free'd*/
-    EOT_DB_NUMBER,      /* Integer */
+    EOT_DB_NUMBER       /* Integer */
 };
 
 enum ea_flags { /* special EA flags */
@@ -585,14 +525,14 @@ enum ea_flags { /* special EA flags */
     EAF_REL         =  8,   /* IP-relative addressing */
     EAF_ABS         = 16,   /* non-IP-relative addressing */
     EAF_FSGS        = 32,   /* fs/gs segment override present */
-    EAF_MIB         = 64,   /* mib operand */
+    EAF_MIB         = 64    /* mib operand */
 };
 
 enum eval_hint { /* values for `hinttype' */
     EAH_NOHINT   = 0,       /* no hint at all - our discretion */
     EAH_MAKEBASE = 1,       /* try to make given reg the base */
     EAH_NOTBASE  = 2,       /* try _not_ to make reg the base */
-    EAH_SUMMED   = 3,       /* base and index are summed into index */
+    EAH_SUMMED   = 3        /* base and index are summed into index */
 };
 
 typedef struct operand { /* operand to an instruction */
@@ -632,7 +572,7 @@ enum ea_type {
     EA_SCALAR,      /* Scalar EA */
     EA_XMMVSIB,     /* XMM vector EA */
     EA_YMMVSIB,     /* YMM vector EA */
-    EA_ZMMVSIB,     /* ZMM vector EA */
+    EA_ZMMVSIB      /* ZMM vector EA */
 };
 
 /*
@@ -676,7 +616,7 @@ enum ttypes {
     QVM   = 015,
     OVM   = 016,
     M128  = 017,
-    DUP   = 020,
+    DUP   = 020
 };
 
 /* EVEX.L'L : Vector length on vector insns */
@@ -684,7 +624,7 @@ enum vectlens {
     VL128 = 0,
     VL256 = 1,
     VL512 = 2,
-    VLMAX = 3,
+    VLMAX = 3
 };
 
 /* If you need to change this, also change it in insns.pl */
@@ -749,13 +689,9 @@ struct ofmt {
     struct dfmt **debug_formats;
 
     /*
-     * and a pointer to the element that is being used
-     * note: this is set to the default at compile time and changed if the
-     * -F option is selected.  If developing a set of new debug formats for
-     * an output format, be sure to set this to whatever default you want
-     *
+     * the default debugging format if -F is not specified
      */
-    const struct dfmt *current_dfmt;
+    const struct dfmt *default_dfmt;
 
     /*
      * This, if non-NULL, is a NULL-terminated list of `char *'s
@@ -907,7 +843,7 @@ struct ofmt {
      * One thing the cleanup routine should always do is to close
      * the output file pointer.
      */
-    void (*cleanup)(int debuginfo);
+    void (*cleanup)(void);
 };
 
 /*
@@ -955,7 +891,7 @@ struct dfmt {
     /*
      * debug_deflabel - called whenever a label is defined. Parameters
      * are the same as to 'symdef()' in the output format. This function
-     * would be called before the output format version.
+     * is called after the output format version.
      */
 
     void (*debug_deflabel)(char *name, int32_t segment, int64_t offset,
