@@ -114,7 +114,7 @@ enum reltype {
     RL_BRANCH,			/* Relative direct branch */
     RL_SUB,			/* X86_64_RELOC_SUBTRACT */
     RL_GOT,			/* X86_64_RELOC_GOT */
-    RL_GOTLOAD,			/* X86_64_RELOC_GOT_LOAD */
+    RL_GOTLOAD			/* X86_64_RELOC_GOT_LOAD */
 };
 #define RL_MAX_32	RL_TLV
 #define RL_MAX_64	RL_GOTLOAD
@@ -300,9 +300,6 @@ static uint64_t rel_padcnt = 0;
 #define alignptr(x) \
     ALIGN(x, fmt.ptrsize)	/* align x to output format width */
 
-static void debug_reloc (struct reloc *);
-static void debug_section_relocs (struct section *) _unused;
-
 static struct section *get_section_by_name(const char *segname,
                                            const char *sectname)
 {
@@ -468,6 +465,8 @@ static int64_t add_reloc(struct section *sect, int32_t section,
 	    r->snum = raa_read(extsyms, section);
 	    if (reltype == RL_BRANCH)
 		r->type = X86_64_RELOC_BRANCH;
+	    else if (r->type == GENERIC_RELOC_VANILLA)
+		adjust = -sect->size;
 	} else {
 	    /* local */
 	    r->ext = 0;
@@ -1142,7 +1141,7 @@ static void macho_calculate_sizes (void)
     }
 
     if (seg_nsects > MAX_SECT) {
-	nasm_error(ERR_FATAL, "MachO output is limited to %d sections\n",
+	nasm_fatal(0, "MachO output is limited to %d sections\n",
 		   MAX_SECT);
     }
 
@@ -1324,6 +1323,8 @@ static void macho_write_section (void)
 		l += sectstab[r->snum]->addr;
 		if (r->pcrel)
 		    l -= s->addr;
+	    } else if (r->pcrel && r->type == GENERIC_RELOC_VANILLA) {
+		l -= s->addr;
 	    }
 
 	    /* write new offset back */
@@ -1546,14 +1547,12 @@ static void macho_write (void)
    for the object file, writing, and then freeing all of the data from
    the file.  */
 
-static void macho_cleanup(int debuginfo)
+static void macho_cleanup(void)
 {
     struct section *s;
     struct reloc *r;
     struct symbol *sym;
 
-    (void)debuginfo;
-
     /* Sort all symbols.  */
     macho_layout_symbols (&nsyms, &strslen);
 
@@ -1595,30 +1594,6 @@ static void macho_cleanup(int debuginfo)
     nasm_free(sectstab);
 }
 
-/* Debugging routines.  */
-static void debug_reloc (struct reloc *r)
-{
-    fprintf (stdout, "reloc:\n");
-    fprintf (stdout, "\taddr: %"PRId32"\n", r->addr);
-    fprintf (stdout, "\tsnum: %d\n", r->snum);
-    fprintf (stdout, "\tpcrel: %d\n", r->pcrel);
-    fprintf (stdout, "\tlength: %d\n", r->length);
-    fprintf (stdout, "\text: %d\n", r->ext);
-    fprintf (stdout, "\ttype: %d\n", r->type);
-}
-
-static void debug_section_relocs (struct section *s)
-{
-    struct reloc *r = s->relocs;
-
-    fprintf (stdout, "relocs for section %s:\n\n", s->sectname);
-
-    while (r != NULL) {
-	debug_reloc (r);
-	r = r->next;
-    }
-}
-
 #ifdef OF_MACHO32
 static const struct macho_fmt macho32_fmt = {
     4,
