@@ -109,7 +109,7 @@ static void int32_ttolocal(int32_t *l)
 #endif
 }
 
-char copybytes(FILE * fp, FILE * fp2, int n)
+static char copybytes(FILE * fp, FILE * fp2, int n)
 {
     int i, t = 0;
 
@@ -129,7 +129,7 @@ char copybytes(FILE * fp, FILE * fp2, int n)
     return (char)t;             /* return last char read */
 }
 
-int32_t copyint32_t(FILE * fp, FILE * fp2)
+static int32_t copyint32_t(FILE * fp, FILE * fp2)
 {
     int32_t l;
     int i, t;
@@ -166,7 +166,7 @@ int main(int argc, char **argv)
 
     if (argc < 3 || !strncmp(argv[1], "-h", 2)
         || !strncmp(argv[1], "--h", 3)) {
-        printf(usage);
+        fputs(usage, stdout);
         exit(1);
     }
 
@@ -180,7 +180,8 @@ int main(int argc, char **argv)
         }
         fwrite(sig_modname, 1, strlen(sig_modname) + 1, fp);
         fwrite(rdl_signature, 1, strlen(rdl_signature), fp);
-        l = sizeof(t = time(NULL));
+	t = time(NULL);
+        l = sizeof(t);
         fwrite(&l, sizeof(l), 1, fp);
         fwrite(&t, 1, l, fp);
         fclose(fp);
